@@ -18,18 +18,19 @@ class S3rverDriver(drivers.Driver):
 
     DEFAULT_PORT = 4568
 
-    def __init__(self, port=DEFAULT_PORT,
-                 **kwargs):
+    def __init__(self, port=DEFAULT_PORT, **kwargs):
+        """Create a new s3rver instance."""
         super(S3rverDriver, self).__init__(**kwargs)
         self.port = port
 
     @classmethod
-    def get_parser(cls, parser):
-        parser.add_argument("--port",
-                            type=int,
-                            default=cls.DEFAULT_PORT,
-                            help="port to use for s3rver")
-        return parser
+    def get_options(cls):
+        return [
+            {"param_decls": ["--port"],
+             "type": int,
+             "default": cls.DEFAULT_PORT,
+             "help": "port to use for s3rver"},
+        ]
 
     def _setUp(self):
         super(S3rverDriver, self)._setUp()
